
import java.util.NoSuchElementException;
import java.util.Vector;

/**
 * Code has been taken from the course textbook: Cormen, Leiserson, Rivest,
 * Stein: Introduction to Algorithm (2nd edition). McGraw-Hill (2001), ISBN:
 * 0070131511
 */

public class AVLTree {

	/** Root of the AVL tree. */
	public Node root;

	private Vector almacenados;

	/**
	 * The constructor creating a binary search tree with just a
	 * <code>null</code>, which is the root.
	 */
	public AVLTree() {
		root = null;
		almacenados = new Vector();
	}

	/**
	 * Searches the tree for a node with a given key. If such a node exists,
	 * prints the value of that node.
	 * 
	 * @param k
	 *            The key being searched for.
	 * @return A reference to a <code>Node</code> object with key
	 *         <code>k</code> if such a node exists. An exception
	 *         NoSuchElementException is thrown if no node exists.
	 */
	public Node search(int k) {
		Node node = search(root, k);
		return node;
	} //search

	/**
	 * Searches the subtree rooted at a given node for a node with a given key.
	 * 
	 * @param node
	 *            Root of the subtree.
	 * @param k
	 *            The key being searched for.
	 * @return A reference to a <code>Node</code> object with key
	 *         <code>k</code> if such a node exists. An exception
	 *         NoSuchElementException is thrown if no node exists.
	 */

	public Node search(Node node, int k) {
		while ((node != null) && (k != node.data.getKey())) {
			if (k < node.data.getKey())
				node = node.left;
			else
				node = node.right;
		} //while

		return node;
	} //search

	/**
	 * Returns the node with the minimum key in the subtree rooted at a node.
	 * 
	 * @param x
	 *            Root of the subtree.
	 * @return A <code>Node</code> object with the minimum key in the tree, or
	 *         the sentinel <code>nil</code> if the tree is empty.
	 */
	protected Node treeMinimum(Node x) {
		while (x.left != null)
			x = x.left;

		return x;
	} //treeMinimum

	/**
	 * Returns the successor of a given node in an inorder walk of the tree.
	 * 
	 * @param node
	 *            The node whose successor is returned.
	 * @return If <code>node</code> has a successor, it is returned.
	 *         Otherwise, return the sentinel <code>null</code>.
	 */
	public Node successor(Node node) {
		Node x = node;

		if (x.right != null)
			return treeMinimum(x.right);

		Node y = x.parent;
		while (y != null && x == y.right) {
			x = y;
			y = y.parent;
		}

		return y;
	} //successor

	/**
	 * Iterates up a tree updating balance factors and performing rotations
	 * after an insert. The method returns when all the necessary updates have
	 * been performed.
	 */
	private void iterateUpAfterInsert(Node iter) {
		//The parent of node iter at the start of each iteration
		Node parent;

		//The new root of the rotated subtree
		Node newRoot;

		//The variable stop is set to true after a rotation or after a balance
		//factor is set to 0. In these cases, no more balance factors need
		//to be updated
		boolean stop = false;

		while ((iter != null) && !stop) {
			parent = iter.parent;

			if (parent == null)
				iter = iter.parent;
			else {
				//Update the parent's balance factor
				if (parent.left == iter) {
					parent.bf--;
					if (parent.bf == 0)
						stop = true;
				} else if (parent.right == iter) {
					parent.bf++;
					if (parent.bf == 0)
						stop = true;
				}

				//Traverse one node up the tree
				iter = iter.parent;
				parent = iter.parent;

				//Perform any necessary rotations
				if ((iter.bf == -2) || (iter.bf == +2)) {
					stop = true;

					if ((iter.bf == -2) && (iter.left.bf == -1))
						newRoot = iter.rotateRight();
					else if ((iter.bf == -2) && (iter.left.bf == +1))
						newRoot = iter.rotateLeftRight();
					else if ((iter.bf == +2) && (iter.right.bf == +1))
						newRoot = iter.rotateLeft();
					else
						newRoot = iter.rotateRightLeft();

					// Update the root pointer if necessary;
					// otherwise, update the pointers between the
					// newRoot and its new parent.
					if (parent == null) {
						root = newRoot;
						newRoot.parent = null;
					} else if (parent.left == iter) {
						parent.left = newRoot;
						newRoot.parent = parent;
					} else {
						parent.right = newRoot;
						newRoot.parent = parent;
					}

				}//if

			}//else
		} //while
	} //iterateUpAfterInsert

	/**
	 * Iterates up a tree updating balance factors and performing rotations
	 * after a delete. The method returns when all the necessary updates have
	 * been performed.
	 */
	private void iterateUpAfterDelete(Node iter) {
		// The variable stop is set to true after the root of a rotated
		// subtree is -1 or +1. In this case, no more balance
		// factors need to be updated
		boolean stop = false;

		//The parent of node inter at the start of each iteration
		Node parent;

		//The new root of the rotated subtree
		Node newRoot;

		// Exit the loop when
		// i) we reached the root (i.e. inter == null)
		// ii) the root of a rotated subtree has a balance factor of
		// -1 or +1 (i.e., stop == true), or
		// iii) the current node has a balance factor of -1 or +1

		// Note: The delete code is more complicated than the insert
		// code, since we may need to do several rotations at different
		// levels of the tree.
		while ((iter != null) && !stop && ((iter.bf != -1) || (iter.bf != +1))) {

			parent = iter.parent;

			//Perform any necessary rotations, and update the
			//parent's balance factors
			if ((iter.bf == -2) || (iter.bf == +2)) {
				if ((iter.bf == -2) && (iter.left.bf != +1))
					newRoot = iter.rotateRight();
				else if ((iter.bf == -2) && (iter.left.bf == +1))
					newRoot = iter.rotateLeftRight();
				else if ((iter.bf == +2) && (iter.right.bf != -1))
					newRoot = iter.rotateLeft();
				else
					newRoot = iter.rotateRightLeft();

				// If the balance factor of the new root is -1 or +1,
				// no more rotations need to be performed

				if ((newRoot.bf == -1) || (newRoot.bf == +1))
					stop = true;

				// Update the root pointer if necessary;
				// otherwise, update the pointers between the
				// newRoot and its new parent.
				if (parent == null) {
					root = newRoot;
					newRoot.parent = null;
				} else if (parent.left == iter) {
					parent.left = newRoot;
					newRoot.parent = parent;
					if (!stop)
						parent.bf++;
				} else {
					parent.right = newRoot;
					newRoot.parent = parent;
					if (!stop)
						parent.bf--;
				}
			}//if

			// No rotations are necessary here. Simply update the
			// parent's balance factor
			else if (parent != null) {
				if (parent.left == iter)
					parent.bf++;
				else
					parent.bf--;
			}//else if

			//Traverse one node up the tree.
			iter = parent;
			if (iter != null)
				parent = iter.parent;

		}//while
	} //iterateUpAfterDelete

	/**
	 * Inserts a data item into the tree, creating a new node for this data.
	 * 
	 * @param data
	 *            Data to be inserted into the tree.
	 * @return A reference to the <code>Node</code> object created.
	 */
	public Node insert(Comparable data) {
		Node z = new Node(data);
		treeInsert(z);
		return z;
	} //insert

	/**
	 * Inserts a node into the tree.
	 * 
	 * @param z
	 *            The node to insert.
	 */
	protected void treeInsert(Node z) {
		Node y = null;
		Node x = root;

		while (x != null) {
			y = x;
			if (z.data.getKey() <= x.data.getKey())
				x = x.left;
			else
				x = x.right;
		} //while

		//Find the location where the node should be inserted
		z.parent = y;
		if (y == null)
			root = z; // the tree had been empty
		else {
			if (z.data.getKey() <= y.data.getKey())
				y.left = z;
			else
				y.right = z;
		}//else

		//Perform any necessary rotations
		iterateUpAfterInsert(z);

	} //treeInsert

	/**
	 * Removes a node with the given key from the tree.
	 * 
	 * @param k
	 *            The key to be removed.
	 */
	public void deleteKey(int k) {
		Node node = (Node) search(root, k);
		if (node != null)
			delete(node);
		else
			throw new NoSuchElementException("Not Found");
	} //deleteKey

	/**
	 * Removes all nodes from the tree.
	 *  
	 */
	public void deleteAllElements() {
		Node aux = root;
		inorden(aux);
		aux = next();
		while (aux != null) {
			delete(aux);
			aux = next();
		}
	}

	/**
	 * Removes a node from the tree.
	 * 
	 * @param node
	 *            The node to be removed. do nothing when deleting a null node
	 *            <code>null</code>.
	 *  
	 */

	public void delete(Node node) {
		Node z = node;

		// Make sure that there is no attempt to delete a null node
		if (z == null)
			throw new NoSuchElementException("empty node");

		Node x; // Replaces z as the subtree's root

		/*
		 * Case 1 and 2: The node to delete has 0 or 1 children. If the node has
		 * no children, we simply delete it. Otherwise, if the node has one
		 * child, we replce the node's contents with the child's contents. In
		 * these two cases, the balance factors of the tree may change.
		 */

		if ((z.left == null) || (z.right == null)) {
			if (z.left == null) // z has no left child
				x = z.right;
			else
				// z has no right child
				x = z.left;

			// Fix links between the parent of the subtree and x, and
			// update the balance factors of the parent of the deleted node
			if (x != null)
				x.parent = z.parent;
			if (root == z)
				root = x;
			else {
				if (z == z.parent.left) {
					z.parent.left = x;
					z.parent.bf++;
				} else {
					z.parent.right = x;
					z.parent.bf--;
				}
			} //else

			// Perform any necessary rotations
			iterateUpAfterDelete(z.parent);

		} //if ((z.left == null) || (z.right == null))

		/*
		 * Case #3: If the node has 2 children, then we splice out the node and
		 * replace it with its successor. Notice that replacing this node does
		 * not change any balance factors.
		 */

		else {
			x = (Node) successor(z); // replace z with its successor
			delete(x); // Delete the node where the successor was

			// Splice out z and put x in its place by fixing links
			// with children.
			x.left = z.left;
			x.right = z.right;
			x.bf = z.bf;
			if (x.left != null)
				x.left.parent = x;
			if (x.right != null)
				x.right.parent = x;

			// Fix links between the parent of the subtree and x.
			if (x != null)
				x.parent = z.parent;
			if (root == z)
				root = x;
			else {
				if (z == z.parent.left)
					z.parent.left = x;
				else
					z.parent.right = x;
			} //else

		} //else

	} //delete

	public void inorden(Node n) {
		if (n != null) {
			inorden(n.left);
			addVector(n);//Lo aado al vector
			inorden(n.right);
		}

	}

	private void addVector(Node n) {
		almacenados.addElement(n);
	}

	public Node next() {
		Node n = null;
		if (almacenados.size() != 0) {
			n = (Node) almacenados.firstElement();
			almacenados.removeElementAt(0);
		}
		return n;
	}
}